/*           
Module : PStat.h
Purpose: Interface for a generalised framework for executing a lengthy operation in a thread. 
         Feedback is provided by a progress dialog which can optionally be cancelled.
Created: 27-03-1997

Copyright (c) 1997 - 2017 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


////////////////////////////// Macros / Defines ///////////////////////////////

#pragma once

#ifndef __PSTAT_H__
#define __PSTAT_H__

#ifndef __AFXMT_H__
#pragma message("To avoid this message please put afxmt.h into your pre compiled header (normally stdafx.h)")
#include <afxmt.h>
#endif //#ifndef __AFXMT_H__

#ifndef CPROGRESSTHREADDLG_EXT_CLASS
#define CPROGRESSTHREADDLG_EXT_CLASS
#endif //#ifndef CPROGRESSTHREADDLG_EXT_CLASS


////////////////////////////// Classes ////////////////////////////////////////


class CPROGRESSTHREADDLG_EXT_CLASS CProgressThreadSettings
{
public:
//Constructors / Destructors
  CProgressThreadSettings() : m_bAllowCancel(false),
                              m_bConfirmCancel(false),
                              m_nMessageBoxType(MB_YESNO),
                              m_nMessageBoxConfirmation(IDYES),
                              m_dwItemData(0),
                              m_nThreadPriority(THREAD_PRIORITY_NORMAL),
                              m_nProgressMinRange(0),
                              m_nProgressMaxRange(100)
  {
  }

//Member variables
  BOOL      m_bAllowCancel;              //Display a cancel button on the progress dialog
  BOOL      m_bConfirmCancel;            //Confirm cancel by displaying a messagebox
  CString   m_sCaption;                  //If set the caption to use on the progress dialog. If empty then the default caption from the dialog resource will be used
  CString   m_sConfirmPrompt;            //The text to use in the messagebox
  int       m_nMessageBoxType;           //The nType value to use for the messagebox. By default this will be MB_YESNO
  int       m_nMessageBoxConfirmation;   //The value from the AfxMessageBox call which will be cancel the dialog. By default this will be IDYES
  DWORD_PTR m_dwItemData;                //The item data which will be used in the function callback
  int       m_nThreadPriority;           //The thread priority to use, by default will be THREAD_PRIORITY_NORMAL
  int       m_nProgressMinRange;         //The minimum value for the progress control, by default will be 0
  int       m_nProgressMaxRange;         //The maximum value for the progress control, by default will be 100
};


class CPROGRESSTHREADDLG_EXT_CLASS CProgressThreadDlg : public CDialog
{
public:
//The declaration of the function pointer to be called in a secondary thread with a progress dialog
  typedef void (FUNCTION_WITH_PROGRESS)(_In_ CProgressThreadDlg* pProgressDlg, _In_opt_ DWORD_PTR dwItemData);

//Constructors / Destructors
  CProgressThreadDlg(_In_ FUNCTION_WITH_PROGRESS* pfnFunction, _In_ const CProgressThreadSettings& settings, _In_opt_ CWnd* pParent = nullptr);
  virtual ~CProgressThreadDlg();

//Methods
  void ReportProgress(_In_ int nCurrentPosition);
  LONG HasBeenCancelled();
  LONG OkToClose();

protected:
//Member variables
  enum { IDD = IDD_PSTAT_PROGRESS };
  CProgressCtrl	                 m_ctrlProgress;
  FILETIME                       m_ftStart;
  int                            m_nCurrentPosition;
  volatile LONG                  m_bOkToClose;
  volatile LONG                  m_bCancelled;
  CWinThread*                    m_pThread;
  FUNCTION_WITH_PROGRESS*        m_pfnFunction;
  void*                          m_pData;
  const CProgressThreadSettings& m_Settings;

//Methods
  virtual void OnCancel();
  virtual void OnOK();
  virtual UINT ProgressDialogWorkerThread();
  static UINT _ProgressDialogWorkerThread(_In_ LPVOID pParam);
  virtual void DoDataExchange(CDataExchange* pDX);
  virtual BOOL OnInitDialog();
  virtual bool GetStartTime();
  virtual bool InitializeDialog();
  virtual bool CreateWorkerThread();

  DECLARE_MESSAGE_MAP()

  DECLARE_DYNAMIC(CProgressThreadDlg)
};

#endif //#ifndef __PSTAT_H__
