#include "stdafx.h"
#include "pstattest.h"
#include "pstattestDlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif

const int PRIMES_TO_CALCULATE = 1000000;


CPStatTestDlg::CPStatTestDlg(_In_opt_ CWnd* pParent) : CDialog(CPStatTestDlg::IDD, pParent),
                                                       m_bAllowCancel(FALSE),
                                                       m_bConfirmCancel(FALSE),
                                                       m_hIcon(AfxGetApp()->LoadIcon(IDR_MAINFRAME))
{
}

void CPStatTestDlg::DoDataExchange(CDataExchange* pDX)
{
  //Let the base class do its thing
  __super::DoDataExchange(pDX);

  DDX_Control(pDX, IDC_CONFIRM_CANCEL, m_ctrlConfirmCancel);
  DDX_Control(pDX, IDC_ALLOW_CANCEL, m_ctrlAllowCancel);
  DDX_Check(pDX, IDC_ALLOW_CANCEL, m_bAllowCancel);
  DDX_Check(pDX, IDC_CONFIRM_CANCEL, m_bConfirmCancel);
}

BEGIN_MESSAGE_MAP(CPStatTestDlg, CDialog)
  ON_WM_PAINT()
  ON_WM_QUERYDRAGICON()
  ON_BN_CLICKED(IDC_WITH_PSTAT, &CPStatTestDlg::OnWithPstat)
  ON_BN_CLICKED(IDC_WITHOUT_PSTAT, &CPStatTestDlg::OnWithoutPstat)
  ON_BN_CLICKED(IDC_ALLOW_CANCEL, &CPStatTestDlg::OnAllowCancel)
  ON_BN_CLICKED(IDC_CONFIRM_CANCEL, &CPStatTestDlg::OnConfirmCancel)
END_MESSAGE_MAP()

BOOL CPStatTestDlg::OnInitDialog()
{
  //Let the base class do its thing
  __super::OnInitDialog();

  OnAllowCancel();

  SetIcon(m_hIcon, TRUE);
  SetIcon(m_hIcon, FALSE);
  
  return TRUE;
}

void CPStatTestDlg::OnPaint() 
{
  if (IsIconic())
  {
    CPaintDC dc(this);
    SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);
    int cxIcon = GetSystemMetrics(SM_CXICON);
    int cyIcon = GetSystemMetrics(SM_CYICON);
    CRect rect;
    GetClientRect(&rect);
    int x = (rect.Width() - cxIcon + 1) / 2;
    int y = (rect.Height() - cyIcon + 1) / 2;
    dc.DrawIcon(x, y, m_hIcon);
  }
  else
  {
    //Let the base class do its thing
    __super::OnPaint();
  }
}

HCURSOR CPStatTestDlg::OnQueryDragIcon()
{
  return static_cast<HCURSOR>(m_hIcon);
}

void CPStatTestDlg::OnWithPstat() 
{
  // Call PStat with the function which we want to execute without hanging the UI
  CProgressThreadSettings settings;
  settings.m_bAllowCancel = m_bAllowCancel ? true : false;
  settings.m_bConfirmCancel = m_bConfirmCancel ? true : false;
  settings.m_sCaption.Format(_T("Calculating the %dth prime number"), PRIMES_TO_CALCULATE);
  settings.m_sConfirmPrompt = _T("Ok to cancel calculation?");
  settings.m_nMessageBoxType = MB_YESNO;
  settings.m_nMessageBoxConfirmation = IDYES;
  settings.m_nThreadPriority = THREAD_PRIORITY_NORMAL;
  long nPrimeNumber = 0;
  settings.m_dwItemData = reinterpret_cast<DWORD_PTR>(&nPrimeNumber);

  CProgressThreadDlg dlg(CalculatePrimeNumbers, settings, this);
  if (dlg.DoModal() == IDOK)
  {
    CString sMsg;
    sMsg.Format(_T("The %dth prime number is %d"), PRIMES_TO_CALCULATE, nPrimeNumber);
    AfxMessageBox(sMsg);
  }
  else
    AfxMessageBox(_T("User cancelled the operation!"));
}

void CPStatTestDlg::OnWithoutPstat() 
{
  //Call our function again, but this time without the use of PStat
  int nPrimeNumber = 0;

  CWaitCursor wait;
  CalculatePrimeNumbers(nullptr, reinterpret_cast<DWORD_PTR>(&nPrimeNumber));

  CString sMsg;
  sMsg.Format(_T("The %dth prime number is %d"), PRIMES_TO_CALCULATE, nPrimeNumber);
  AfxMessageBox(sMsg);
}

void CPStatTestDlg::OnAllowCancel() 
{
  //Get the current value of m_bAllowCancel from the UI
  CDataExchange DX(this, TRUE);
  DDX_Check(&DX, IDC_ALLOW_CANCEL, m_bAllowCancel);

  if (!m_bAllowCancel)
  {
    //Update the member variable
    m_bConfirmCancel = FALSE;

    //Update the UI
    CDataExchange DX2(this, FALSE);
    DDX_Check(&DX, IDC_ALLOW_CANCEL, m_bAllowCancel);
  }
  m_ctrlConfirmCancel.EnableWindow(m_bAllowCancel);
}

void CPStatTestDlg::OnConfirmCancel() 
{
  //Get the current value of m_bAllowCancel from the UI
  CDataExchange DX(this, TRUE);
  DDX_Check(&DX, IDC_CONFIRM_CANCEL, m_bConfirmCancel);
}

BOOL CPStatTestDlg::IsPrime(_In_ long nNumber) 
{
  if ((nNumber % 2) == 0)
    return FALSE;

  for (long i=3; (i*i)<=nNumber; i+=2)
  {
    if ((nNumber % i) == 0)
      return FALSE;
  }

  return TRUE;
}

void CPStatTestDlg::CalculatePrimeNumbers(_In_opt_ CProgressThreadDlg* pDlg, _In_ DWORD_PTR dwItemData)
{
  long i = 1;
  long nNumPrimes = 0;
  while (nNumPrimes <= PRIMES_TO_CALCULATE)
  {
    if (IsPrime(i))
      ++nNumPrimes;

    i += 2;

    if (pDlg != nullptr)
    {
      //Return immediately if user requested it
      if (pDlg->HasBeenCancelled())
        return;

      //othewise update the current percentage done
      pDlg->ReportProgress(nNumPrimes * 100 / PRIMES_TO_CALCULATE);
    }
  }

  long* pnResult = reinterpret_cast<long*>(dwItemData);
  AFXASSUME(pnResult != nullptr);
  *pnResult = i;
}
